local LubStub = LibStub
local ClassicCountdown = LibStub('AceAddon-3.0'):NewAddon('ClassicCountdown', 'AceTimer-3.0', 'AceEvent-3.0', 'AceConsole-3.0')
_G.ClassicCountdown = ClassicCountdown

local SML = LibStub("LibSharedMedia-3.0");

local CLASSIC_RELEASE = 1566856801

local updateTime
local lastUpdateTime = GetServerTime()

ClassicCountdown.frame = nil

local function GetFormatTime(value)

	if value <= 0 then return -1,-1,-1,-1 end

	local SECONDS_IN_HOUR = 60 * 60
	local SECONDS_IN_DAY = 24 * SECONDS_IN_HOUR

	local secondsLeft = value
	
	local days = math.floor((secondsLeft / SECONDS_IN_DAY))
	secondsLeft = secondsLeft - (days * SECONDS_IN_DAY)

	local hours = math.floor((secondsLeft / SECONDS_IN_HOUR) % 24)
	secondsLeft = secondsLeft - (hours * SECONDS_IN_HOUR)

	local minutes = math.floor((secondsLeft / 60) % 60)
	secondsLeft = secondsLeft - (minutes * 60)

	local seconds = secondsLeft % 60
	return days, hours, minutes, seconds
end


local function LongFormatTime(value)
	local days, hours, minutes, seconds = GetFormatTime(value)
	if days == -1 then 
		return "!!!RELEASED!!!"
	else
		return format('%d days %d hours %d minutes %d seconds', days, hours, minutes, seconds);
	end
end

local function ShortFormatTime(value)
	local days, hours, minutes, seconds = GetFormatTime(value)
	if days == -1 then 
		return "!!!RELEASED!!!"
	else
		return format('%dd %dh:%dm:%ds', days, hours, minutes, seconds)
	end
end


function ClassicCountdown:OnInitialize()
	self:RegisterOptions()
	self:RegisterChatCommand("cc", "SlashCommand")
	self:RegisterChatCommand("ClassicCountdown", "SlashCommand")
end

function ClassicCountdown:SlashCommand(info)
	if ClassicCountdown.optionsDialog then
		if InterfaceOptionsFrame:IsVisible() then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(ClassicCountdown.optionsDialog) 
		end
	end
end

local ClassicCountdown_Events = {}
local eventFrame = CreateFrame("Frame", nil, WorldFrame)

local function DoEvent(self, event, ...)
	if ClassicCountdown_Events[event] then
		ClassicCountdown_Events[event](event, ...)
	end
end

function DoUpdate()
	updateTime = GetServerTime()
	if ClassicCountdown.db.profile.enabled and updateTime > lastUpdateTime then
		local diffTime = CLASSIC_RELEASE - updateTime
		local timeText = LongFormatTime(diffTime)
		if not ClassicCountdown.db.profile.longFormat then
			timeText = ShortFormatTime(diffTime)
		end
		ClassicCountdown:UpdateClockFrame(timeText)
		lastUpdateTime = updateTime
	end
end

eventFrame:SetFrameStrata("TOOLTIP") 
eventFrame:SetScript("OnEvent", DoEvent)
eventFrame:SetScript("OnUpdate",DoUpdate)

function ClassicCountdown_Events:PLAYER_ENTERING_WORLD()
	if not ClassicCountdown.frame then 
		ClassicCountdown.frame = ClassicCountdown:SetupFrame()
	end
end

for eventname in pairs(ClassicCountdown_Events) do 
	eventFrame:RegisterEvent(eventname) 
end

-- Options
local registry = LibStub('AceConfigRegistry-3.0')

local function RestoreDefaults(info)
	ClassicCountdown.db.profile = GetDefaults()
	ClassicCountdown:ApplyFrameSettings(ClassicCountdown.frame)
end

local function GetValue(info)
	return ClassicCountdown.db.profile[info.arg]
end

local function SetValue(info, value)
	ClassicCountdown.db.profile[info.arg] = value
	ClassicCountdown:ApplyFrameSettings(ClassicCountdown.frame)
end

function GetGeneralOptionsTable()
	return {
		name = 'Classic Countdown',
		type = 'group',
		set = SetValue,
		get = GetValue,
		handler = ClassicCountdown,
		args = {
			["Enable"] = {
				name = "Enable",
				type = 'toggle',
				order = 100,
				width = 'half',
				arg = 'enabled',
				disabled = false,
			},
			["Format"] = {
				name = "Use long time format",
				type = 'toggle',
				width = 'full',
				order = 110,
				arg = 'longFormat',
				disabled = false,
			},
			["BREAK1"] = {
				name = "",
				type = 'description',
				order = 150,
			},
			["Scale"] = {
				name = "Change the scale of the clock",
				type = 'range',
				width = 'double',
				min = .1, max = 2, step = .1,
				order = 160,
				arg = 'scale',
			},
		},	
	}
end


function GetDefaults()
	local defaults = {
		profile = {
			enabled = true,
			longFormat = true,
			scale = 1,
			
			x = 0,
			p1 = "CENTER",
			p2 = "CENTER",
			y = 0,
			anchorFrame = "UIParent",
		}
	}
	return defaults
end

function ClassicCountdown:ProfileChanged()
	self:ApplyFrameSettings(ClassicCountdown.frame)
end

function ClassicCountdown:RegisterOptions()
 	self.db = LibStub('AceDB-3.0'):New('ClassicCountdownDB', GetDefaults(), 'Default')
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	self:RegisterMedia()
	
	local profileOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	local dialog = LibStub('AceConfigDialog-3.0')
	registry:RegisterOptionsTable("Classic Countdown", {name = "",type = 'group',desc = "",args = {}})
	local top = dialog:AddToBlizOptions("Classic Countdown", "Classic Countdown")
	top:SetScript('OnShow', function(self)
		InterfaceOptionsFrame_OpenToCategory(ClassicCountdown.optionsDialog)
	end)
	
	registry:RegisterOptionsTable('Classic Countdown - General', GetGeneralOptionsTable)
	ClassicCountdown.optionsDialog = dialog:AddToBlizOptions('Classic Countdown - General', 'General',"Classic Countdown")
	
	registry:RegisterOptionsTable("Classic Countdown Profile", profileOptions)
	dialog:AddToBlizOptions("Classic Countdown Profile", "Profile", "Classic Countdown")
	
	self:ProfileChanged()
end

-- Clock functions

function ClassicCountdown:GetAnchorFrame()
	return "CENTER","UIParent","CENTER",0,0
end

function ClassicCountdown:ApplyFrameSettings(frame)
	if not frame then return end -- should only really happen on OnInitialize

	local settings = ClassicCountdown.db.profile

	if settings.enabled == nil then 
		settings.enabled = true
	end

	if not settings.anchorFrame then
		settings.p1, settings.anchorFrame, settings.p2, settings.x, settings.y = ClassicCountdown:GetAnchorFrame()
	end


	frame:ClearAllPoints()
	frame:SetPoint(settings.p1,settings.anchorFrame,settings.p2,settings.x,settings.y)
	
	if settings.scale then 
		frame:SetScale(settings.scale)
	end

	if not settings.enabled then
		frame:Hide()
	end
end

local function ShowToolTip(frame)
	if not ClassicCountdown.db.profile.lock then
		GameTooltip:SetOwner(frame, "ANCHOR_NONE");
		local x, y = GetCursorPosition();
		local scale = GameTooltip:GetEffectiveScale();
		GameTooltip:ClearAllPoints();
		GameTooltip:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", (x/scale + 20), (y/scale - 20));
		GameTooltip:SetText( "|cff00ccffClassic Countdown|r|cff00ccff Unlocked|r\n   |cffffffffDrag|r to reposition\n   |cffffffffRight click|r to lock"  )
		if frame.isDragging then
			GameTooltip:Hide()
		else
			GameTooltip:Show()
		end
	end
end


local function HideToolTip(frame)
	GameTooltip:Hide()
end


function ClassicCountdown:SetupFrame()
	print("Setting up frame")
	local frame = CreateFrame("Button", "ClassicCountdownFrame", UIParent)
	frame:SetFrameStrata("HIGH")
	--frame.texture = frame:CreateTexture("bg", "BACKGROUND", "ActionBarButtonTemplate")
	--frame.texture:SetAllPoints(frame)
	--frame.texture:SetTexture(0,0,0,.8)

	frame.titleText2 = frame:CreateFontString("ClassicCountdown-TitleText2", "BACKGROUND")
	frame.titleText2:SetPoint("TOP",frame, 0, -13)

	frame.titleText2:SetFont(SML:Fetch('font', 'LifeCraft'),24,"OUTLINE")
	frame.titleText2:SetTextColor(.8,.6,0)
	frame.titleText2:SetJustifyH("CENTER")
	frame.titleText2:SetText("COUNTDOWN")

	frame.titleText1 = frame:CreateFontString("ClassicCountdown-TitleText1", "OVERLAY")
	frame.titleText1:SetPoint("TOP",frame, 0, 0)
	
	frame.titleText1:SetFont(SML:Fetch('font', 'Folkard'),14,"OUTLINE")
	frame.titleText1:SetTextColor(1,.8,0)
	frame.titleText1:SetJustifyH("CENTER")
	frame.titleText1:SetText("CLASSIC")

	frame.time = frame:CreateFontString("ClassicCountdown-TimeText", "OVERLAY")
	frame.time:SetPoint("TOP", frame, 0, -40)
	frame.time:SetFontObject(GameFontWhite)
	frame.time:SetJustifyH("CENTER")

	local width = frame.time:GetStringWidth()
	local height = 45+frame.time:GetStringHeight() 
	local titleWidth = frame.titleText2:GetStringWidth()

	if titleWidth < width then width = titleWidth end

	frame:SetHeight(60)
	frame:SetWidth(width+10)

	frame:SetClampedToScreen(true)
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForClicks("RightButtonUp")
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", 
		function(frame) 
			if not ClassicCountdown.db.profile.lock then
				frame:StartMoving() 
				frame.isDragging = true
				HideToolTip(frame)
			end
		end
	)
	frame:SetScript("OnDragStop", 
		function(frame) 
			frame:StopMovingOrSizing() 
			local settings = ClassicCountdown.db.profile
			local p1,_,p2,x,y = frame:GetPoint()
			settings.x = x
			settings.y = y
			settings.anchorFrame = "UIParent"
			settings.p1 = p1
			settings.p2 = p2
			frame.isDragging = false
			ShowToolTip(frame)
		end
	)
	frame:SetScript("OnClick",
		function(frame)
			local settings = ClassicCountdown.db.profile

			if not settings.lock then
				settings.lock = true
			else
				settings.lock = false
			end
			ClassicCountdown:ApplyFrameSettings(frame)
		end
	)
	frame:SetScript("OnEnter", ShowToolTip)
	frame:SetScript("OnLeave", HideToolTip)
	
	self:ApplyFrameSettings(frame)
	return frame
end

function ClassicCountdown:UpdateClockFrame(timeText)
	local clockSettings = ClassicCountdown.db.profile
	local frame = self.frame or self:SetupFrame()

	frame.time:SetText(timeText)

	if clockSettings.enabled or not clockSettings.lock then
		frame:Show()
	elseif frame then
		frame:Hide()
	end
end

function ClassicCountdown:RegisterMedia()
	SML:Register("font", "Folkard",		[[Interface\Addons\ClassicCountdown\fonts\folkard_.ttf]])
	SML:Register("font", "LifeCraft",	[[Interface\Addons\ClassicCountdown\fonts\LifeCraft_Font.ttf]])
end